# AUTOGENERATED! DO NOT EDIT! File to edit: ../../../nbs/ml.llm.dataset.ipynb.

# %% auto 0
__all__ = ['alpaca2sharegpt', 'alpaca2openai', 'sharegpt_from2openai_role', 'openai_role2sharegpt_from', 'sharegpt2openai']

# %% ../../../nbs/ml.llm.dataset.ipynb 3
def alpaca2sharegpt(example):
    messages = []
    if "instruction" in example:
        messages.append({"from": "system", "value": example["instruction"]})
    messages.append({"from": "human", "value": example["input"]})
    messages.append({"from": "gpt", "value": example["output"]})
    return {"conversations": messages}


def alpaca2openai(example):
    messages = []
    if "instruction" in example:
        messages.append({"role": "system", "content": example["instruction"]})
    messages.append({"role": "user", "content": example["input"]})
    messages.append({"role": "assistant", "content": example["output"]})
    return {"conversations": messages}


def sharegpt_from2openai_role(message):
    return {"system": "system", "human": "user", "gpt": "assistant"}[message["from"]]


def openai_role2sharegpt_from(message):
    return {"system": "system", "user": "human", "assistant": "gpt"}[message["role"]]


def sharegpt2openai(example):
    return {
        "conversations": [
            {"role": sharegpt_from2openai_role(message), "content": message["value"]}
            for message in example["conversations"]
        ],
    }



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/ml.wandb.ipynb.

# %% auto 0
__all__ = ['log', 'fetch_wandb_run', 'fetch_wandb_config']

# %% ../../nbs/ml.wandb.ipynb 3
import wandb
from ..logging import get_logger

log = get_logger(__name__)

# %% ../../nbs/ml.wandb.ipynb 4
def fetch_wandb_run(*, entity: str, project: str, run_id: str):
    return wandb.Api().run(f"{entity}/{project}/{run_id}")

def fetch_wandb_config(*, entity: str, project: str, run_id: str):
    run = fetch_wandb_run(entity=entity, project=project, run_id=run_id)
    return run.config



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/bellek',
                'doc_host': 'https://bdsaglam.github.io',
                'git_url': 'https://github.com/bdsaglam/bellek',
                'lib_path': 'bellek'},
  'syms': { 'bellek.logging': {'bellek.logging.get_logger': ('logging.html#get_logger', 'bellek/logging.py')},
            'bellek.ml.clip': { 'bellek.ml.clip.ClipClassificationHead': ('ml.clip.html#clipclassificationhead', 'bellek/ml/clip.py'),
                                'bellek.ml.clip.ClipClassificationHead.__init__': ( 'ml.clip.html#clipclassificationhead.__init__',
                                                                                    'bellek/ml/clip.py'),
                                'bellek.ml.clip.ClipClassificationHead.forward': ( 'ml.clip.html#clipclassificationhead.forward',
                                                                                   'bellek/ml/clip.py'),
                                'bellek.ml.clip.ClipZeroShotClassifier': ('ml.clip.html#clipzeroshotclassifier', 'bellek/ml/clip.py'),
                                'bellek.ml.clip.ClipZeroShotClassifier.__init__': ( 'ml.clip.html#clipzeroshotclassifier.__init__',
                                                                                    'bellek/ml/clip.py'),
                                'bellek.ml.clip.ClipZeroShotClassifier.compute_text_features': ( 'ml.clip.html#clipzeroshotclassifier.compute_text_features',
                                                                                                 'bellek/ml/clip.py'),
                                'bellek.ml.clip.ClipZeroShotClassifier.forward': ( 'ml.clip.html#clipzeroshotclassifier.forward',
                                                                                   'bellek/ml/clip.py'),
                                'bellek.ml.clip.load_clip_preprocess': ('ml.clip.html#load_clip_preprocess', 'bellek/ml/clip.py'),
                                'bellek.ml.clip.make_tfms_from_clip_preprocess': ( 'ml.clip.html#make_tfms_from_clip_preprocess',
                                                                                   'bellek/ml/clip.py')},
            'bellek.ml.coop': { 'bellek.ml.coop.ClipVisualEncoder': ('ml.coop.html#clipvisualencoder', 'bellek/ml/coop.py'),
                                'bellek.ml.coop.ClipVisualEncoder.__init__': ( 'ml.coop.html#clipvisualencoder.__init__',
                                                                               'bellek/ml/coop.py'),
                                'bellek.ml.coop.ClipVisualEncoder.forward': ('ml.coop.html#clipvisualencoder.forward', 'bellek/ml/coop.py'),
                                'bellek.ml.coop.PromptLearner': ('ml.coop.html#promptlearner', 'bellek/ml/coop.py'),
                                'bellek.ml.coop.PromptLearner.__init__': ('ml.coop.html#promptlearner.__init__', 'bellek/ml/coop.py'),
                                'bellek.ml.coop.PromptLearner.forward': ('ml.coop.html#promptlearner.forward', 'bellek/ml/coop.py'),
                                'bellek.ml.coop.PromptLearningClip': ('ml.coop.html#promptlearningclip', 'bellek/ml/coop.py'),
                                'bellek.ml.coop.PromptLearningClip.__init__': ( 'ml.coop.html#promptlearningclip.__init__',
                                                                                'bellek/ml/coop.py'),
                                'bellek.ml.coop.PromptLearningClip.forward': ( 'ml.coop.html#promptlearningclip.forward',
                                                                               'bellek/ml/coop.py'),
                                'bellek.ml.coop.PromptLearningTextEncoder': ('ml.coop.html#promptlearningtextencoder', 'bellek/ml/coop.py'),
                                'bellek.ml.coop.PromptLearningTextEncoder.__init__': ( 'ml.coop.html#promptlearningtextencoder.__init__',
                                                                                       'bellek/ml/coop.py'),
                                'bellek.ml.coop.PromptLearningTextEncoder.forward': ( 'ml.coop.html#promptlearningtextencoder.forward',
                                                                                      'bellek/ml/coop.py'),
                                'bellek.ml.coop.TextEncoder': ('ml.coop.html#textencoder', 'bellek/ml/coop.py'),
                                'bellek.ml.coop.TextEncoder.__init__': ('ml.coop.html#textencoder.__init__', 'bellek/ml/coop.py'),
                                'bellek.ml.coop.TextEncoder.forward': ('ml.coop.html#textencoder.forward', 'bellek/ml/coop.py'),
                                'bellek.ml.coop.make_prompt_learning_clip': ('ml.coop.html#make_prompt_learning_clip', 'bellek/ml/coop.py'),
                                'bellek.ml.coop.prepare_prompt_learning_clip': ( 'ml.coop.html#prepare_prompt_learning_clip',
                                                                                 'bellek/ml/coop.py')},
            'bellek.ml.data': { 'bellek.ml.data.ZippedDataLoader': ('ml.data.html#zippeddataloader', 'bellek/ml/data.py'),
                                'bellek.ml.data.ZippedDataLoader.__init__': ('ml.data.html#zippeddataloader.__init__', 'bellek/ml/data.py'),
                                'bellek.ml.data.ZippedDataLoader.__iter__': ('ml.data.html#zippeddataloader.__iter__', 'bellek/ml/data.py'),
                                'bellek.ml.data.ZippedDataLoader.__next__': ('ml.data.html#zippeddataloader.__next__', 'bellek/ml/data.py'),
                                'bellek.ml.data.ZippedDataset': ('ml.data.html#zippeddataset', 'bellek/ml/data.py'),
                                'bellek.ml.data.ZippedDataset.__getitem__': ('ml.data.html#zippeddataset.__getitem__', 'bellek/ml/data.py'),
                                'bellek.ml.data.ZippedDataset.__init__': ('ml.data.html#zippeddataset.__init__', 'bellek/ml/data.py'),
                                'bellek.ml.data.ZippedDataset.__len__': ('ml.data.html#zippeddataset.__len__', 'bellek/ml/data.py'),
                                'bellek.ml.data.fetch_imagenet_label_synsets': ( 'ml.data.html#fetch_imagenet_label_synsets',
                                                                                 'bellek/ml/data.py'),
                                'bellek.ml.data.get_imagenet_id_label_map': ('ml.data.html#get_imagenet_id_label_map', 'bellek/ml/data.py'),
                                'bellek.ml.data.get_imagenet_id_synset_map': ( 'ml.data.html#get_imagenet_id_synset_map',
                                                                               'bellek/ml/data.py'),
                                'bellek.ml.data.make_imagenet_id_synset_map': ( 'ml.data.html#make_imagenet_id_synset_map',
                                                                                'bellek/ml/data.py')},
            'bellek.ml.evaluation': { 'bellek.ml.evaluation.classification_summary': ( 'ml.evaluation.html#classification_summary',
                                                                                       'bellek/ml/evaluation.py'),
                                      'bellek.ml.evaluation.evaluate_slmc': ('ml.evaluation.html#evaluate_slmc', 'bellek/ml/evaluation.py'),
                                      'bellek.ml.evaluation.find_binary_clf_threshold': ( 'ml.evaluation.html#find_binary_clf_threshold',
                                                                                          'bellek/ml/evaluation.py')},
            'bellek.ml.experiment': { 'bellek.ml.experiment.main': ('ml.experiment.html#main', 'bellek/ml/experiment.py'),
                                      'bellek.ml.experiment.make_experiment_dir': ( 'ml.experiment.html#make_experiment_dir',
                                                                                    'bellek/ml/experiment.py'),
                                      'bellek.ml.experiment.make_run_experiment_sweep': ( 'ml.experiment.html#make_run_experiment_sweep',
                                                                                          'bellek/ml/experiment.py'),
                                      'bellek.ml.experiment.prepare_config': ( 'ml.experiment.html#prepare_config',
                                                                               'bellek/ml/experiment.py')},
            'bellek.ml.kg.cons': { 'bellek.ml.kg.cons.ERX2AlpacaFormatter': ('ml.kg.cons.html#erx2alpacaformatter', 'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.ERX2AlpacaFormatter.__post_init__': ( 'ml.kg.cons.html#erx2alpacaformatter.__post_init__',
                                                                                            'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.ERX2AlpacaFormatter._choose_few_shot_examples': ( 'ml.kg.cons.html#erx2alpacaformatter._choose_few_shot_examples',
                                                                                                        'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.ERX2AlpacaFormatter.format': ( 'ml.kg.cons.html#erx2alpacaformatter.format',
                                                                                     'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.ERX2AlpacaFormatter.make_system_prompt': ( 'ml.kg.cons.html#erx2alpacaformatter.make_system_prompt',
                                                                                                 'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.ERX2ChatFormatter': ('ml.kg.cons.html#erx2chatformatter', 'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.ERX2ChatFormatter.__post_init__': ( 'ml.kg.cons.html#erx2chatformatter.__post_init__',
                                                                                          'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.ERX2ChatFormatter._choose_few_shot_examples': ( 'ml.kg.cons.html#erx2chatformatter._choose_few_shot_examples',
                                                                                                      'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.ERX2ChatFormatter.format': ( 'ml.kg.cons.html#erx2chatformatter.format',
                                                                                   'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.ERX2ChatFormatter.make_messages': ( 'ml.kg.cons.html#erx2chatformatter.make_messages',
                                                                                          'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.ERX2ChatFormatter.make_system_message': ( 'ml.kg.cons.html#erx2chatformatter.make_system_message',
                                                                                                'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.evaluate_joint_er_extraction': ( 'ml.kg.cons.html#evaluate_joint_er_extraction',
                                                                                       'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.evaluate_joint_er_extractions': ( 'ml.kg.cons.html#evaluate_joint_er_extractions',
                                                                                        'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.evaluate_model_jer': ('ml.kg.cons.html#evaluate_model_jer', 'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.evaluate_pipe_jer': ('ml.kg.cons.html#evaluate_pipe_jer', 'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.format_few_shot_example': ( 'ml.kg.cons.html#format_few_shot_example',
                                                                                  'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.format_few_shot_examples': ( 'ml.kg.cons.html#format_few_shot_examples',
                                                                                   'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.format_triplets': ('ml.kg.cons.html#format_triplets', 'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.parse_triplet_strings': ( 'ml.kg.cons.html#parse_triplet_strings',
                                                                                'bellek/ml/kg/cons.py'),
                                   'bellek.ml.kg.cons.parse_triplets': ('ml.kg.cons.html#parse_triplets', 'bellek/ml/kg/cons.py')},
            'bellek.ml.kg.dataset': { 'bellek.ml.kg.dataset._batch_transform_webnlg': ( 'ml.kg.dataset.html#_batch_transform_webnlg',
                                                                                        'bellek/ml/kg/dataset.py'),
                                      'bellek.ml.kg.dataset._transform_entity': ( 'ml.kg.dataset.html#_transform_entity',
                                                                                  'bellek/ml/kg/dataset.py'),
                                      'bellek.ml.kg.dataset._transform_relation': ( 'ml.kg.dataset.html#_transform_relation',
                                                                                    'bellek/ml/kg/dataset.py'),
                                      'bellek.ml.kg.dataset._transform_triplet': ( 'ml.kg.dataset.html#_transform_triplet',
                                                                                   'bellek/ml/kg/dataset.py'),
                                      'bellek.ml.kg.dataset.batch_transform_webnlg': ( 'ml.kg.dataset.html#batch_transform_webnlg',
                                                                                       'bellek/ml/kg/dataset.py')},
            'bellek.ml.lang.dataset': { 'bellek.ml.lang.dataset.alpaca2openai': ( 'ml.lang.dataset.html#alpaca2openai',
                                                                                  'bellek/ml/lang/dataset.py'),
                                        'bellek.ml.lang.dataset.alpaca2sharegpt': ( 'ml.lang.dataset.html#alpaca2sharegpt',
                                                                                    'bellek/ml/lang/dataset.py'),
                                        'bellek.ml.lang.dataset.openai2sharegpt': ( 'ml.lang.dataset.html#openai2sharegpt',
                                                                                    'bellek/ml/lang/dataset.py'),
                                        'bellek.ml.lang.dataset.openai2sharegpt_message': ( 'ml.lang.dataset.html#openai2sharegpt_message',
                                                                                            'bellek/ml/lang/dataset.py'),
                                        'bellek.ml.lang.dataset.sharegpt2openai': ( 'ml.lang.dataset.html#sharegpt2openai',
                                                                                    'bellek/ml/lang/dataset.py'),
                                        'bellek.ml.lang.dataset.sharegpt2openai_message': ( 'ml.lang.dataset.html#sharegpt2openai_message',
                                                                                            'bellek/ml/lang/dataset.py')},
            'bellek.ml.layer': { 'bellek.ml.layer.GradReverse': ('ml.layer.html#gradreverse', 'bellek/ml/layer.py'),
                                 'bellek.ml.layer.GradReverse.__init__': ('ml.layer.html#gradreverse.__init__', 'bellek/ml/layer.py'),
                                 'bellek.ml.layer.GradReverse.forward': ('ml.layer.html#gradreverse.forward', 'bellek/ml/layer.py'),
                                 'bellek.ml.layer.reverse_grad': ('ml.layer.html#reverse_grad', 'bellek/ml/layer.py'),
                                 'bellek.ml.layer.reverse_grad.backward': ('ml.layer.html#reverse_grad.backward', 'bellek/ml/layer.py'),
                                 'bellek.ml.layer.reverse_grad.forward': ('ml.layer.html#reverse_grad.forward', 'bellek/ml/layer.py')},
            'bellek.ml.llama': { 'bellek.ml.llama.prepare_llama2_for_inference': ( 'ml.llama.html#prepare_llama2_for_inference',
                                                                                   'bellek/ml/llama.py'),
                                 'bellek.ml.llama.prepare_llama2_for_training': ( 'ml.llama.html#prepare_llama2_for_training',
                                                                                  'bellek/ml/llama.py')},
            'bellek.ml.llama_index': { 'bellek.ml.llama_index.HuggingFaceTextGenInferenceLLM': ( 'ml.llama_index.html#huggingfacetextgeninferencellm',
                                                                                                 'bellek/ml/llama_index.py')},
            'bellek.ml.llm.dataset': { 'bellek.ml.llm.dataset.alpaca2openai': ( 'ml.llm.dataset.html#alpaca2openai',
                                                                                'bellek/ml/llm/dataset.py'),
                                       'bellek.ml.llm.dataset.alpaca2sharegpt': ( 'ml.llm.dataset.html#alpaca2sharegpt',
                                                                                  'bellek/ml/llm/dataset.py'),
                                       'bellek.ml.llm.dataset.openai_role2sharegpt_from': ( 'ml.llm.dataset.html#openai_role2sharegpt_from',
                                                                                            'bellek/ml/llm/dataset.py'),
                                       'bellek.ml.llm.dataset.sharegpt2openai': ( 'ml.llm.dataset.html#sharegpt2openai',
                                                                                  'bellek/ml/llm/dataset.py'),
                                       'bellek.ml.llm.dataset.sharegpt_from2openai_role': ( 'ml.llm.dataset.html#sharegpt_from2openai_role',
                                                                                            'bellek/ml/llm/dataset.py')},
            'bellek.ml.loss': { 'bellek.ml.loss.BinaryFocalLoss': ('ml.loss.html#binaryfocalloss', 'bellek/ml/loss.py'),
                                'bellek.ml.loss.BinaryFocalLoss.__init__': ('ml.loss.html#binaryfocalloss.__init__', 'bellek/ml/loss.py'),
                                'bellek.ml.loss.BinaryFocalLoss.activation': ( 'ml.loss.html#binaryfocalloss.activation',
                                                                               'bellek/ml/loss.py'),
                                'bellek.ml.loss.BinaryFocalLoss.decodes': ('ml.loss.html#binaryfocalloss.decodes', 'bellek/ml/loss.py'),
                                'bellek.ml.loss.BinaryFocalLoss.forward': ('ml.loss.html#binaryfocalloss.forward', 'bellek/ml/loss.py')},
            'bellek.ml.mcd': { 'bellek.ml.mcd.DiscrepancyLoss': ('ml.mcd.html#discrepancyloss', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.DiscrepancyLoss.__call__': ('ml.mcd.html#discrepancyloss.__call__', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.EnsembleMcdModel': ('ml.mcd.html#ensemblemcdmodel', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.EnsembleMcdModel.__init__': ('ml.mcd.html#ensemblemcdmodel.__init__', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.EnsembleMcdModel.forward': ('ml.mcd.html#ensemblemcdmodel.forward', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.EnsembleMcdModel.from_mcd_model': ( 'ml.mcd.html#ensemblemcdmodel.from_mcd_model',
                                                                                  'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.Feature': ('ml.mcd.html#feature', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.Feature.__init__': ('ml.mcd.html#feature.__init__', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.Feature.forward': ('ml.mcd.html#feature.forward', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdCallback': ('ml.mcd.html#mcdcallback', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdCallback.__init__': ('ml.mcd.html#mcdcallback.__init__', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdCallback._do_grad_opt': ('ml.mcd.html#mcdcallback._do_grad_opt', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdCallback._do_one_batch': ('ml.mcd.html#mcdcallback._do_one_batch', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdCallback._predict_source': ('ml.mcd.html#mcdcallback._predict_source', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdCallback._predict_target': ('ml.mcd.html#mcdcallback._predict_target', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdCallback.before_batch': ('ml.mcd.html#mcdcallback.before_batch', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdCallback.before_fit': ('ml.mcd.html#mcdcallback.before_fit', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdDataLoader': ('ml.mcd.html#mcddataloader', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdDataLoader.__init__': ('ml.mcd.html#mcddataloader.__init__', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdDataLoader.__iter__': ('ml.mcd.html#mcddataloader.__iter__', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdDataLoader.__len__': ('ml.mcd.html#mcddataloader.__len__', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdDataLoader.__next__': ('ml.mcd.html#mcddataloader.__next__', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdDataLoader.new': ('ml.mcd.html#mcddataloader.new', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdDataLoader.to': ('ml.mcd.html#mcddataloader.to', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdDataset': ('ml.mcd.html#mcddataset', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdDataset.__getitem__': ('ml.mcd.html#mcddataset.__getitem__', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdDataset.__init__': ('ml.mcd.html#mcddataset.__init__', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdDataset.__len__': ('ml.mcd.html#mcddataset.__len__', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdModel': ('ml.mcd.html#mcdmodel', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdModel.__init__': ('ml.mcd.html#mcdmodel.__init__', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.McdModel.forward': ('ml.mcd.html#mcdmodel.forward', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.Predictor': ('ml.mcd.html#predictor', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.Predictor.__init__': ('ml.mcd.html#predictor.__init__', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.Predictor.forward': ('ml.mcd.html#predictor.forward', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.discrepancy': ('ml.mcd.html#discrepancy', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.discrepancy_metric': ('ml.mcd.html#discrepancy_metric', 'bellek/ml/mcd.py'),
                               'bellek.ml.mcd.mcd_learner': ('ml.mcd.html#mcd_learner', 'bellek/ml/mcd.py')},
            'bellek.ml.transformers': { 'bellek.ml.transformers.load_tokenizer_model': ( 'ml.transformers.html#load_tokenizer_model',
                                                                                         'bellek/ml/transformers.py'),
                                        'bellek.ml.transformers.merge_adapters_and_publish': ( 'ml.transformers.html#merge_adapters_and_publish',
                                                                                               'bellek/ml/transformers.py'),
                                        'bellek.ml.transformers.preprocess_config': ( 'ml.transformers.html#preprocess_config',
                                                                                      'bellek/ml/transformers.py')},
            'bellek.ml.vision': { 'bellek.ml.vision.Make3Channel': ('ml.vision.html#make3channel', 'bellek/ml/vision.py'),
                                  'bellek.ml.vision.Make3Channel.__call__': ('ml.vision.html#make3channel.__call__', 'bellek/ml/vision.py'),
                                  'bellek.ml.vision.Make3Channel.__repr__': ('ml.vision.html#make3channel.__repr__', 'bellek/ml/vision.py'),
                                  'bellek.ml.vision.TorchVisionTransform': ('ml.vision.html#torchvisiontransform', 'bellek/ml/vision.py'),
                                  'bellek.ml.vision.TorchVisionTransform.__init__': ( 'ml.vision.html#torchvisiontransform.__init__',
                                                                                      'bellek/ml/vision.py'),
                                  'bellek.ml.vision.TorchVisionTransform.encodes': ( 'ml.vision.html#torchvisiontransform.encodes',
                                                                                     'bellek/ml/vision.py')},
            'bellek.ml.wandb': { 'bellek.ml.wandb.fetch_wandb_config': ('ml.wandb.html#fetch_wandb_config', 'bellek/ml/wandb.py'),
                                 'bellek.ml.wandb.fetch_wandb_run': ('ml.wandb.html#fetch_wandb_run', 'bellek/ml/wandb.py')},
            'bellek.testing': { 'bellek.testing.fixture_directory': ('testing.html#fixture_directory', 'bellek/testing.py'),
                                'bellek.testing.fixture_path': ('testing.html#fixture_path', 'bellek/testing.py')},
            'bellek.utils': { 'bellek.utils.NestedDict': ('utils.html#nesteddict', 'bellek/utils.py'),
                              'bellek.utils.NestedDict.__init__': ('utils.html#nesteddict.__init__', 'bellek/utils.py'),
                              'bellek.utils.NestedDict.at': ('utils.html#nesteddict.at', 'bellek/utils.py'),
                              'bellek.utils.NestedDict.flat': ('utils.html#nesteddict.flat', 'bellek/utils.py'),
                              'bellek.utils.NestedDict.from_flat_dict': ('utils.html#nesteddict.from_flat_dict', 'bellek/utils.py'),
                              'bellek.utils.NestedDict.set': ('utils.html#nesteddict.set', 'bellek/utils.py'),
                              'bellek.utils.Path.ls_sorted': ('utils.html#path.ls_sorted', 'bellek/utils.py'),
                              'bellek.utils.context_chdir': ('utils.html#context_chdir', 'bellek/utils.py'),
                              'bellek.utils.flatten_dict': ('utils.html#flatten_dict', 'bellek/utils.py'),
                              'bellek.utils.generate_time_id': ('utils.html#generate_time_id', 'bellek/utils.py'),
                              'bellek.utils.most_common': ('utils.html#most_common', 'bellek/utils.py'),
                              'bellek.utils.split_camel_case': ('utils.html#split_camel_case', 'bellek/utils.py'),
                              'bellek.utils.unflatten_dict': ('utils.html#unflatten_dict', 'bellek/utils.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../../nbs/ml.kg.cons.ipynb.

# %% auto 0
__all__ = ['log', 'Entity', 'Relation', 'Triplet', 'DEFAULT_SYSTEM_PROMPT_TEMPLATE', 'DEFAULT_RELATION_SET_PROMPT_TEMPLATE',
           'DEFAULT_FEW_SHOT_EXAMPLES_PROMPT_TEMPLATE', 'DEFAULT_SYSTEM_PROMPT_TEMPLATE2',
           'DEFAULT_RELATION_SET_PROMPT_TEMPLATE2', 'evaluate_joint_er_extraction', 'evaluate_joint_er_extractions',
           'parse_triplet_strings', 'parse_triplets', 'format_triplets', 'format_few_shot_example',
           'format_few_shot_examples', 'ERX2AlpacaFormatter', 'ERX2ChatFormatter', 'evaluate_pipe_jer',
           'evaluate_model_jer']

# %% ../../../nbs/ml.kg.cons.ipynb 3
import random
from dataclasses import dataclass
from typing import TypeAlias, Iterable, Callable, Any, Generator
import numpy as np

from ...logging import get_logger

log = get_logger(__name__)

# %% ../../../nbs/ml.kg.cons.ipynb 5
Entity: TypeAlias = str|tuple[str, str]
Relation: TypeAlias = str
Triplet: TypeAlias = tuple[Entity, Relation, Entity]

# %% ../../../nbs/ml.kg.cons.ipynb 6
def evaluate_joint_er_extraction(*, reference: Iterable[Triplet], prediction: Iterable[Triplet]):
    """
    Example: [(('John', 'PERSON'), 'works_at', ('Google', 'ORG'))]
    """

    reference_set = set(reference)
    prediction_set = set(prediction)
    assert len(reference) == len(reference_set), "Duplicates found in references"

    TP = len(reference_set & prediction_set)
    FP = len(prediction_set - reference_set)
    FN = len(reference_set - prediction_set)
    
    # Calculate metrics
    precision = TP / (TP + FP) if TP + FP > 0 else 0
    recall = TP / (TP + FN) if TP + FN > 0 else 0
    f1_score = 2 * (precision * recall) / (precision + recall) if precision + recall > 0 else 0
    
    return {
        'precision': precision,
        'recall': recall,
        'f1': f1_score
    }

def evaluate_joint_er_extractions(*, references: Iterable[Iterable[Triplet]], predictions: Iterable[Iterable[Triplet]]):
    score_dicts = [
        evaluate_joint_er_extraction(reference=reference, prediction=prediction) 
        for reference, prediction in zip(references, predictions)
    ]
    return {('mean_' + key): np.mean([scores[key] for scores in score_dicts]) for key in score_dicts[0].keys()}

# %% ../../../nbs/ml.kg.cons.ipynb 9
def parse_triplet_strings(text: str, delimiter: str="|") -> list[str]:
    return [line for line in text.splitlines() if line and line.count(delimiter) == 2]

def parse_triplets(text: str, delimiter: str="|") -> list[Triplet]:
    return [tuple(triplet_string.split(delimiter)) for triplet_string in parse_triplet_strings(text, delimiter=delimiter)]

# %% ../../../nbs/ml.kg.cons.ipynb 12
def format_triplets(triplets: Iterable[str]) -> str:
    return '\n'.join(triplets)

def format_few_shot_example(example, text_prefix="# Text\n", triplets_prefix="# Triplets\n"):
    text = example['text']
    triplets = format_triplets(example['triplets'])
    return f"{text_prefix}{text}\n{triplets_prefix}{triplets}"

def format_few_shot_examples(examples):
    return "\n\n".join([format_few_shot_example(example) for example in examples])

# %% ../../../nbs/ml.kg.cons.ipynb 13
DEFAULT_SYSTEM_PROMPT_TEMPLATE = """You are a helpful assistant that extracts entity-relation-entity triplets from given text.
{relation_set_prompt}
{few_shot_prompt}
""".strip()

DEFAULT_RELATION_SET_PROMPT_TEMPLATE = """Here are the list of relations that you can use:
{relation_set}
"""

DEFAULT_FEW_SHOT_EXAMPLES_PROMPT_TEMPLATE = """Use the same format for triplets as in the examples provided below.
{few_shot_examples}
"""

@dataclass
class ERX2AlpacaFormatter:
    system_prompt_template: str = DEFAULT_SYSTEM_PROMPT_TEMPLATE
    relation_set_prompt_template: str = DEFAULT_RELATION_SET_PROMPT_TEMPLATE
    relation_set: set|None = None
    few_shot_examples_prompt_template: str = DEFAULT_FEW_SHOT_EXAMPLES_PROMPT_TEMPLATE
    few_shot_examples: list[dict]|None = None
    n_few_shot_examples: int = 3

    def __post_init__(self):
        if self.relation_set:
            self.relation_set = sorted(self.relation_set)
        if self.few_shot_examples is not None:
            self.few_shot_examples = list(self.few_shot_examples)

    def format(self, example: dict):
        instruction = self.make_system_prompt()
        input = example['text']
        output = '\n'.join(example['triplets'])
        return {
            'instruction': instruction.strip(),
            'input': input.strip(),
            'output': output.strip(),
        }

    def make_system_prompt(self) -> str:
        rsp = self.relation_set_prompt_template.format(relation_set=','.join(self.relation_set)) if self.relation_set else ""
        fsp = self.few_shot_examples_prompt_template.format(few_shot_examples=format_few_shot_examples(self._choose_few_shot_examples())) if self.few_shot_examples else ""
        return self.system_prompt_template.format(relation_set_prompt=rsp, few_shot_prompt=fsp)

    def _choose_few_shot_examples(self) -> list[dict]:
        if len(self.few_shot_examples) <= self.n_few_shot_examples:
            return self.few_shot_examples
        else:
            return random.sample(self.few_shot_examples, k=self.n_few_shot_examples)

# %% ../../../nbs/ml.kg.cons.ipynb 16
DEFAULT_SYSTEM_PROMPT_TEMPLATE2 = """You are a helpful assistant that extracts entity-relation-entity triplets from given text. Use '|' as delimiter and provide one triplet per line.
{relation_set_prompt}
""".strip()

DEFAULT_RELATION_SET_PROMPT_TEMPLATE2 = """Here are the list of relations that you can use:
{relation_set}
""".strip()

@dataclass
class ERX2ChatFormatter:
    system_prompt_template: str = DEFAULT_SYSTEM_PROMPT_TEMPLATE2
    relation_set_prompt_template: str = DEFAULT_RELATION_SET_PROMPT_TEMPLATE2
    relation_set: set|None = None
    few_shot_examples: list[dict]|None = None
    n_few_shot_examples: int = 3

    def __post_init__(self):
        if self.relation_set:
            self.relation_set = sorted(self.relation_set)
        if self.few_shot_examples is not None:
            self.few_shot_examples = list(self.few_shot_examples)

    def format(self, example: dict):
        messages = [
            {"role": "system", "content": self.make_system_message()},
            *list(self.make_messages(*self._choose_few_shot_examples(), example)),
        ]
        return {'conversations': messages}

    def make_system_message(self) -> str:
        rsp = self.relation_set_prompt_template.format(relation_set=','.join(self.relation_set)) if self.relation_set else ""
        return self.system_prompt_template.format(relation_set_prompt=rsp)

    def make_messages(self, *examples) -> Generator[dict, None, None]:
        for example in examples:
            yield {"role": "user", "content": example["text"]}
            yield {"role": "assistant", "content": format_triplets(example["triplets"])}

    def _choose_few_shot_examples(self) -> list[dict]:
        if len(self.few_shot_examples) <= self.n_few_shot_examples:
            return self.few_shot_examples
        else:
            return random.sample(self.few_shot_examples, k=self.n_few_shot_examples)

# %% ../../../nbs/ml.kg.cons.ipynb 19
def evaluate_pipe_jer(dataset, pipe):
    import evaluate

    log.info(f"Evaluating model for JER on dataset with {len(dataset)} samples.")

    results = pipe(dataset["input"])
    generations = [result[0]["generated_text"] for result in results]
    predictions = [parse_triplet_strings(text.strip()) for text in generations]
    references = [parse_triplet_strings(text.strip()) for text in dataset["output"]]
    
    dataf = dataset.to_pandas()
    dataf["generation"] = generations
    dataf["prediction"] = predictions
    dataf["reference"] = references
    
    metric = evaluate.load("bdsaglam/jer")
    scores = metric.compute(predictions=predictions, references=references)

    return scores, dataf

def evaluate_model_jer(
    dataset,
    *,
    response_template: str,
    tokenizer,
    model,
    max_new_tokens=256,
    batch_size=4,
    **kwargs,
):
    assert len(dataset) > 0, "Dataset is empty!"

    def extract_input_output(example):
        input, output = example["text"].rsplit(response_template, 1)
        input += response_template
        return {"input": input, "output": output}

    dataset = dataset.map(extract_input_output)

    # setup generation pipeline
    from transformers import pipeline

    pipe = pipeline(
        task="text-generation",
        model=model,
        tokenizer=tokenizer,
        max_new_tokens=max_new_tokens,
        batch_size=batch_size,
        return_full_text=False,
        **kwargs,
    )

    return evaluate_pipe_jer(dataset, pipe)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/logging.ipynb.

# %% auto 0
__all__ = ['get_logger']

# %% ../nbs/logging.ipynb 3
import logging

# %% ../nbs/logging.ipynb 4
def get_logger(name: str | None = None) -> logging.Logger:
    if name is None:
        name = __name__
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)

    # Create a console handler and set the log level
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.INFO)

    # Create a formatter and add it to the handlers
    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    console_handler.setFormatter(formatter)

    # Add the handlers to the logger
    logger.addHandler(console_handler)

    return logger



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/ml.transformers.ipynb.

# %% auto 0
__all__ = ['log', 'merge_adapters_and_publish', 'load_tokenizer_model', 'preprocess_config']

# %% ../../nbs/ml.transformers.ipynb 3
from copy import deepcopy
import torch
from transformers import AutoModelForCausalLM, AutoTokenizer
from ..utils import NestedDict
from ..logging import get_logger

log = get_logger(__name__)

# %% ../../nbs/ml.transformers.ipynb 4
def merge_adapters_and_publish(
    model_id: str,
    torch_dtype=torch.float16,
    device_map={"": 0},
    merged_model_id: str=None,
):
    from peft import AutoPeftModelForCausalLM
    
    if isinstance(torch_dtype, str) and torch_dtype != "auto":
        torch_dtype = getattr(torch, torch_dtype) 

    log.info(f"Loading model and tokenizer for {model_id}")
    model = AutoPeftModelForCausalLM.from_pretrained(
        model_id,
        torch_dtype=torch_dtype,
        device_map=device_map,
    )
    tokenizer = AutoTokenizer.from_pretrained(model_id, trust_remote_code=True)

    log.info("Merging adapters to model...")
    model = model.merge_and_unload()

    if merged_model_id is None:
        merged_model_id = f"{model_id}-merged"
    log.info(f"Pushing merged model to HF hub as {merged_model_id}")
    model.push_to_hub(merged_model_id)
    tokenizer.push_to_hub(merged_model_id)
    return merged_model_id

# %% ../../nbs/ml.transformers.ipynb 5
def load_tokenizer_model(
    model_name_or_path: str,
    *,
    auto_model_cls=AutoModelForCausalLM,
    device_map={"": 0},
    **model_kwargs,
):
    # Setup quantization config
    if (quantization_config := model_kwargs.get("quantization_config")) and isinstance(quantization_config, dict):
        from transformers import BitsAndBytesConfig
        model_kwargs["quantization_config"] = BitsAndBytesConfig(**quantization_config)
    # Setup torch dtype
    if (torch_dtype := model_kwargs.get("torch_dtype")) and (torch_dtype != "auto"):
        model_kwargs["torch_dtype"] = getattr(torch, torch_dtype)
    # Load model
    model = auto_model_cls.from_pretrained(
        model_name_or_path,
        device_map=device_map,
        **model_kwargs,
    )
    # Load tokenizer
    if auto_model_cls == AutoModelForCausalLM:
        tokenizer_id = model_name_or_path
    else:
        from peft import AutoPeftModelForCausalLM
        if auto_model_cls == AutoPeftModelForCausalLM:
            tokenizer_id = model.active_peft_config.base_model_name_or_path
        else:
            raise ValueError(f"Unknown auto_model_cls: {auto_model_cls}")
    tokenizer = AutoTokenizer.from_pretrained(tokenizer_id, trust_remote_code=True)
    return tokenizer, model

# %% ../../nbs/ml.transformers.ipynb 6
def preprocess_config(config: NestedDict):
    config = deepcopy(config)

    # Set float precision
    major, _ = torch.cuda.get_device_capability()
    if major >= 8:
        log.info("GPU supports bfloat16.")
        torch_dtype, bf16, fp16, bnb_4bit_compute_dtype = ("bfloat16", True, False, "bfloat16")
    else:
        log.info("GPU does not support bfloat16.")
        torch_dtype, bf16, fp16, bnb_4bit_compute_dtype = ("float16", False, True, "float16")

    if config.at("pretrained_model.torch_dtype"):
        config.set("pretrained_model.torch_dtype", torch_dtype)
    if config.at("pretrained_model.quantization_config.load_in_4bit"):
        config.set("pretrained_model.quantization_config.bnb_4bit_compute_dtype", bnb_4bit_compute_dtype)
    if config.at("trainer.training_args.bf16") or config.at("trainer.training_args.fp16"):
        config.set("trainer.training_args.bf16", bf16)
        config.set("trainer.training_args.fp16", fp16)
    
    return config
